"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const package_reader_1 = require("./package-reader");
const fs = require("fs");
const path = require("path");
const match_star_1 = require("./match-star");
/**
 * Creates a function that can resolve paths according to tsconfig paths property.
 * @param tsConfigPath The paths where tsconfig.json is located.
 * @param baseUrl The baseUrl specified in tsconfig.
 * @param paths The paths specified in tsconfig.
 */
function createMatchPath(absoluteBaseUrl, paths) {
    // Resolve all paths to absolute form once here, this saves time on each request later.
    // We also add the baseUrl as a base which will be replace if specified in paths. This is how typescript works
    const absolutePaths = Object.keys(paths).reduce((soFar, key) => (Object.assign({}, soFar, { [key]: paths[key].map(pathToResolve => path.join(absoluteBaseUrl, pathToResolve)) })), {
        "*": [`${absoluteBaseUrl.replace(/\/$/, "")}/*`]
    });
    return (sourceFileName, requestedModule, readPackageJson, fileExists, extensions) => matchFromAbsolutePaths(absolutePaths, sourceFileName, requestedModule, readPackageJson, fileExists, extensions);
}
exports.createMatchPath = createMatchPath;
/**
 * Finds a path from tsconfig that matches a module load request.
 * @param absolutePathMappings The paths to try as specified in tsconfig but resolved to absolute form.
 * @param absoluteSourceFileName Absolute path to the file that requested the module.
 * @param requestedModule The required module name.
 * @param readPackageJson Function that returns parsed package.json if exists or undefined(useful for testing).
 * @param fileExists Function that checks for existance of a file (useful for testing).
 * @param extensions File extensions to probe for (useful for testing).
 * @returns the found path, or undefined if no path was found.
 */
function matchFromAbsolutePaths(absolutePathMappings, absoluteSourceFileName, requestedModule, 
    // tslint:disable-next-line:no-any
    readPackageJson = (packageJsonPath) => package_reader_1.readPackage(packageJsonPath), fileExists = fs.existsSync, extensions = Object.keys(require.extensions)) {
    if (requestedModule[0] !== "." &&
        requestedModule[0] !== path.sep &&
        absolutePathMappings &&
        absoluteSourceFileName &&
        requestedModule &&
        fileExists) {
        for (const virtualPathPattern of sortByLongestPrefix(Object.keys(absolutePathMappings))) {
            const starMatch = virtualPathPattern === requestedModule
                ? ""
                : match_star_1.matchStar(virtualPathPattern, requestedModule);
            if (starMatch !== undefined) {
                for (const physicalPathPattern of absolutePathMappings[virtualPathPattern]) {
                    const physicalPath = physicalPathPattern.replace("*", starMatch);
                    const resolved = tryResolve(physicalPath, fileExists, readPackageJson, extensions);
                    if (resolved) {
                        return resolved;
                    }
                }
            }
        }
    }
    return undefined;
}
exports.matchFromAbsolutePaths = matchFromAbsolutePaths;
/**
 * Tries to resolve a physical path by:
 * 1. Check for files named as last part of request and ending in any of the extensions.
 * 2. Check for file specified in package.json's main property.
 * 3. Check for a file named index ending in any of the extensions.
 * @param physicalPath The path to check.
 * @param fileExists Function that checks for existance of a file (useful for testing).
 * @param readPackageJson Function that returns parsed package.json if exists or undefined(useful for testing).
 * @param extensions File extensions to probe for (useful for testing).
 * @returns {string}
 */
function tryResolve(physicalPath, 
    // tslint:disable-next-line:no-any
    fileExists, 
    // tslint:disable-next-line:no-any
    readPackageJson, extensions) {
    if (path.extname(path.basename(physicalPath)).length > 0 &&
        fileExists(physicalPath)) {
        return physicalPath;
    }
    if (extensions.reduce((prev, curr) => prev || fileExists(physicalPath + curr), false)) {
        return physicalPath;
    }
    const packageJson = readPackageJson(path.join(physicalPath, "/package.json"));
    if (packageJson &&
        packageJson.main &&
        fileExists(path.join(physicalPath, packageJson.main))) {
        const file = path.join(physicalPath, packageJson.main);
        const fileExtension = path.extname(file).replace(/^\./, "");
        const fileExtensionRegex = new RegExp(`\.${fileExtension}$`);
        return fileExtension ? file.replace(fileExtensionRegex, "") : file;
    }
    const indexPath = path.join(physicalPath, "/index");
    return extensions.reduce((prev, curr) => prev || (fileExists(indexPath + curr) && physicalPath), "");
}
/**
 * Sort path patterns.
 * If module name can be matches with multiple patterns then pattern with the longest prefix will be picked.
 */
function sortByLongestPrefix(arr) {
    return arr
        .concat()
        .sort((a, b) => getPrefixLength(b) - getPrefixLength(a));
}
function getPrefixLength(pattern) {
    const prefixLength = pattern.indexOf("*");
    return pattern.substr(0, prefixLength).length;
}
//# sourceMappingURL=match-path.js.map