"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const match_path_1 = require("../src/match-path");
const path_1 = require("path");
describe("match-path", () => {
    it("should locate path that matches with star and exists", () => {
        const matchPath = match_path_1.createMatchPath("/root/", { "lib/*": ["location/*"] });
        const result = matchPath("/root/test.ts", "lib/mylib", (_) => undefined, (name) => name === path_1.join("/root", "location", "mylib", "index.ts"));
        chai_1.assert.equal(result, path_1.join("/root", "location", "mylib"));
    });
    it("should resolve to correct path when many are specified", () => {
        const matchPath = match_path_1.createMatchPath("/root/", {
            "lib/*": ["foo1/*", "foo2/*", "location/*", "foo3/*"]
        });
        const result = matchPath("/root/test.ts", "lib/mylib", (_) => undefined, (name) => name === path_1.join("/root", "location", "mylib", "index.ts"), [".ts"]);
        chai_1.assert.equal(result, path_1.join("/root", "location", "mylib"));
    });
    it("should locate path that matches with star and prioritize pattern with longest prefix", () => {
        const matchPath = match_path_1.createMatchPath("/root/", {
            "*": ["location/*"],
            "lib/*": ["location/*"]
        });
        const result = matchPath("/root/test.ts", "lib/mylib", undefined, (name) => name === path_1.join("/root", "location", "lib", "mylib", "index.ts") ||
            name === path_1.join("/root", "location", "mylib", "index.ts"));
        chai_1.assert.equal(result, path_1.join("/root", "location", "mylib"));
    });
    it("should locate path that matches with star and exists with extension", () => {
        const matchPath = match_path_1.createMatchPath("/root/", { "lib/*": ["location/*"] });
        const result = matchPath("/root/test.ts", "lib/mylib", (_) => undefined, (name) => name === path_1.join("/root", "location", "mylib.myext"), [".js", ".myext"]);
        chai_1.assert.equal(result, path_1.join("/root", "location", "mylib"));
    });
    it("should resolve request with extension specified", () => {
        const matchPath = match_path_1.createMatchPath("/root/", { "lib/*": ["location/*"] });
        const result = matchPath("/root/test.ts", "lib/test.jpg", (_) => undefined, (name) => name === path_1.join("/root", "location", "test.jpg"), [".js", ".myext"]);
        chai_1.assert.equal(result, path_1.join("/root", "location", "test.jpg"));
    });
    it("should locate path that matches without star and exists", () => {
        const matchPath = match_path_1.createMatchPath("/root/", {
            "lib/foo": ["location/foo"]
        });
        const result = matchPath("/root/test.ts", "lib/foo", (_) => undefined, (name) => name === path_1.join("/root", "location", "foo.ts"));
        chai_1.assert.equal(result, path_1.join("/root", "location", "foo"));
    });
    it("should resolve to parent folder when filename is in subfolder", () => {
        const matchPath = match_path_1.createMatchPath("/root/", { "lib/*": ["location/*"] });
        const result = matchPath("/root/subfolder/file.ts", "lib/mylib", (_) => undefined, (name) => name === path_1.join("/root", "location", "mylib", "index.ts"));
        chai_1.assert.equal(result, path_1.join("/root", "location", "mylib"));
    });
    it("should resolve from main field in package.json", () => {
        const matchPath = match_path_1.createMatchPath("/root/", { "lib/*": ["location/*"] });
        const result = matchPath("/root/subfolder/file.ts", "lib/mylib", (_) => ({ main: "./kalle.ts" }), (name) => name === path_1.join("/root", "location", "mylib", "kalle.ts"));
        chai_1.assert.equal(result, path_1.join("/root", "location", "mylib", "kalle"));
    });
    it("should resolve from main field in package.json and correctly remove file extension", () => {
        const matchPath = match_path_1.createMatchPath("/root/", { "lib/*": ["location/*"] });
        const result = matchPath("/root/subfolder/file.js", "lib/mylib.js", (_) => ({ main: "./kalle.js" }), (name) => name === path_1.join("/root", "location", "mylib.js", "kalle.js"), [".ts", ".js"]);
        // Make sure we escape the "."
        const result2 = matchPath("/root/subfolder/file.js", "lib/mylibjs", (_) => ({ main: "./kallejs" }), (name) => name === path_1.join("/root", "location", "mylibjs", "kallejs"), [".ts", ".js"]);
        chai_1.assert.equal(result, path_1.join("/root", "location", "mylib.js", "kalle"));
        chai_1.assert.equal(result2, path_1.join("/root", "location", "mylibjs", "kallejs"));
    });
    it("should resolve to with the help of baseUrl when not explicitly set", () => {
        const matchPath = match_path_1.createMatchPath("/root/", {});
        const result = matchPath("/root/test.ts", "mylib", (_) => undefined, (name) => name === path_1.join("/root", "mylib", "index.ts"));
        chai_1.assert.equal(result, "/root/mylib");
    });
    it("should not locate path that does not match", () => {
        const matchPath = match_path_1.createMatchPath("/root/", { "lib/*": ["location/*"] });
        const result = matchPath("/root/asd.ts", "mylib", (_) => undefined, (name) => name === path_1.join("root", "location", "mylib"));
        chai_1.assert.equal(result, undefined);
    });
});
//# sourceMappingURL=match-path-tests.js.map