"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const tsconfig_loader_1 = require("../src/tsconfig-loader");
const path_1 = require("path");
describe("tsconfig-loader", () => {
    it("should find tsconfig in cwd", () => {
        const result = tsconfig_loader_1.tsConfigLoader({
            cwd: "/foo/bar",
            getEnv: (_) => undefined,
            loadSync: (cwd) => {
                return {
                    tsConfigPath: `${cwd}/tsconfig.json`,
                    baseUrl: "./",
                    paths: {}
                };
            }
        });
        chai_1.assert.equal(result.tsConfigPath, "/foo/bar/tsconfig.json");
    });
    it("should return loaderResult.tsConfigPath as undefined when not found", () => {
        const result = tsconfig_loader_1.tsConfigLoader({
            cwd: "/foo/bar",
            getEnv: (_) => undefined,
            loadSync: (_) => {
                return {
                    tsConfigPath: undefined,
                    baseUrl: "./",
                    paths: {}
                };
            }
        });
        chai_1.assert.isUndefined(result.tsConfigPath);
    });
    it("should use TS_NODE_PROJECT env if exists", () => {
        const result = tsconfig_loader_1.tsConfigLoader({
            cwd: "/foo/bar",
            getEnv: (key) => key === "TS_NODE_PROJECT" ? "/foo/baz" : undefined,
            loadSync: (cwd, fileName) => {
                if (cwd === "/foo/bar" && fileName === "/foo/baz") {
                    return {
                        tsConfigPath: "/foo/baz/tsconfig.json",
                        baseUrl: "./",
                        paths: {}
                    };
                }
                return {
                    tsConfigPath: undefined,
                    baseUrl: "./",
                    paths: {}
                };
            }
        });
        chai_1.assert.equal(result.tsConfigPath, "/foo/baz/tsconfig.json");
    });
});
describe("walkForTsConfig", () => {
    it("should find tsconfig in starting directory", () => {
        const pathToTsconfig = path_1.join("/root", "dir1", "tsconfig.json");
        const res = tsconfig_loader_1.walkForTsConfig(path_1.join("/root", "dir1"), path => path === pathToTsconfig);
        chai_1.assert.equal(res, pathToTsconfig);
    });
    it("should find tsconfig in parent directory", () => {
        const pathToTsconfig = path_1.join("/root", "tsconfig.json");
        const res = tsconfig_loader_1.walkForTsConfig(path_1.join("/root", "dir1"), path => path === pathToTsconfig);
        chai_1.assert.equal(res, pathToTsconfig);
    });
    it("should return undefined when reaching the top", () => {
        const res = tsconfig_loader_1.walkForTsConfig(path_1.join("/root", "dir1", "kalle"), () => false);
        chai_1.assert.equal(res, undefined);
    });
});
describe("loadConfig", () => {
    it("It should load a config", () => {
        const config = { compilerOptions: { baseUrl: "hej" } };
        const res = tsconfig_loader_1.loadTsconfig("/root/dir1/tsconfig.json", path => path === "/root/dir1/tsconfig.json", _ => JSON.stringify(config));
        chai_1.assert.deepEqual(res, config);
    });
    it("It should load a config with comments", () => {
        const config = { compilerOptions: { baseUrl: "hej" } };
        const res = tsconfig_loader_1.loadTsconfig("/root/dir1/tsconfig.json", path => path === "/root/dir1/tsconfig.json", _ => `{
          // my comment
          "compilerOptions": { 
            "baseUrl": "hej"
          }
        }`);
        chai_1.assert.deepEqual(res, config);
    });
    it("It should load a config with extends and overwrite baseUrl", () => {
        const firstConfig = {
            extends: "../base-config.json",
            compilerOptions: { baseUrl: "kalle" }
        };
        const firstConfigPath = path_1.join("/root", "dir1", "tsconfig.json");
        const baseConfig = {
            compilerOptions: { baseUrl: "olle", paths: { foo: ["bar"] } }
        };
        const baseConfigPath = path_1.join("/root", "base-config.json");
        const res = tsconfig_loader_1.loadTsconfig(path_1.join("/root", "dir1", "tsconfig.json"), path => path === firstConfigPath || path === baseConfigPath, path => {
            if (path === firstConfigPath) {
                return JSON.stringify(firstConfig);
            }
            if (path === baseConfigPath) {
                return JSON.stringify(baseConfig);
            }
            return "";
        });
        chai_1.assert.deepEqual(res, {
            extends: "../base-config.json",
            compilerOptions: { baseUrl: "kalle", paths: { foo: ["bar"] } }
        });
    });
    it("Should use baseUrl relative to location of extended tsconfig", () => {
        const firstConfig = { compilerOptions: { baseUrl: "." } };
        const firstConfigPath = path_1.join("/root", "first-config.json");
        const secondConfig = { extends: "../first-config.json" };
        const secondConfigPath = path_1.join("/root", "dir1", "second-config.json");
        const thirdConfig = { extends: "../second-config.json" };
        const thirdConfigPath = path_1.join("/root", "dir1", "dir2", "third-config.json");
        const res = tsconfig_loader_1.loadTsconfig(path_1.join("/root", "dir1", "dir2", "third-config.json"), path => path === firstConfigPath ||
            path === secondConfigPath ||
            path === thirdConfigPath, path => {
            if (path === firstConfigPath) {
                return JSON.stringify(firstConfig);
            }
            if (path === secondConfigPath) {
                return JSON.stringify(secondConfig);
            }
            if (path === thirdConfigPath) {
                return JSON.stringify(thirdConfig);
            }
            return "";
        });
        chai_1.assert.deepEqual(res, {
            extends: "../second-config.json",
            compilerOptions: { baseUrl: path_1.join("..", "..") }
        });
    });
});
//# sourceMappingURL=tsconfig-loader-tests.js.map